/**
 * OW2 Util
 * Copyright (C) 2008 SERLI
 * Contact: easybeans@objectweb.org
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
* USA
*
* --------------------------------------------------------------------------
* $Id: PlanReaderImpl.java 5072 2009-07-28 07:05:56Z fornacif $
* --------------------------------------------------------------------------
*/


package org.ow2.util.plan.reader.plan;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;

import javax.xml.XMLConstants;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.transform.Source;
import javax.xml.transform.sax.SAXSource;
import javax.xml.validation.SchemaFactory;

import org.ow2.util.plan.bindings.deploymentplan.DeploymentPlan;
import org.ow2.util.plan.reader.AbsReader;
import org.ow2.util.plan.reader.ReaderException;
import org.xml.sax.InputSource;

/**
 * Implementation for the IPlanReader interface.
 * @author Mickaël LEDUQUE
 */
public class PlanReaderImpl extends AbsReader implements IPlanReader {

    /**
     * location of the deployment plan schema resources.
     */
    private static final String DEPLOYMENT_PLAN_XSD = "deployment-plan-1.0.xsd";
    private static final String MAVEN2_DEPLOYMENT_PLAN_XSD = "maven2-deployment-plan-1.0.xsd";
    private static final String URL_DEPLOYMENT_PLAN_XSD = "url-deployment-plan-1.0.xsd";
    private static final String OBR_DEPLOYMENT_PLAN_XSD = "obr-deployment-plan-1.0.xsd";

    /**
     * XML binding classes used by PlanReaders.
     */
    private static final Class[] PLAN_CLASSES = {
            org.ow2.util.plan.bindings.deploymentplan.ObjectFactory.class,
            org.ow2.util.plan.bindings.deploymentplan.maven2.ObjectFactory.class,
            org.ow2.util.plan.bindings.deploymentplan.obr.ObjectFactory.class,
            org.ow2.util.plan.bindings.deploymentplan.url.ObjectFactory.class };

    /**
     * {@inheritDoc}
     */
    public PlanReaderImpl() throws Exception {
        super();
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected void initUnmarshallClasses() {
        unmarshallClasses = PLAN_CLASSES;
    }

    /**
     * {@inheritDoc}
     */
    public synchronized DeploymentPlan readPlan(final File file) throws IOException,
            ReaderException {

        JAXBElement jaxbElement = null;
        try {
            jaxbElement = getRootElement(file);
        } catch (JAXBException e) {
            throw new ReaderException("Unable to parse the file '" + file + "'", e);
        }

        Object elementValue = jaxbElement.getValue();
        if (elementValue instanceof DeploymentPlan) {
            return (DeploymentPlan) elementValue;
        } else {
            throw new ReaderException("Bad root element");
        }

    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected void buildUnmarshallerSchema() throws Exception {
        SchemaFactory sf = SchemaFactory.newInstance(XMLConstants.W3C_XML_SCHEMA_NS_URI);
        // Build the composite schema for validation.

        InputStream planSchemaStream = this.schemaStreamFactory.getSchemaStream(DEPLOYMENT_PLAN_XSD);
        InputStream m2SchemaStream = this.schemaStreamFactory.getSchemaStream(MAVEN2_DEPLOYMENT_PLAN_XSD);
        InputStream urlSchemaStream = this.schemaStreamFactory.getSchemaStream(URL_DEPLOYMENT_PLAN_XSD);
        InputStream obrSchemaStream = this.schemaStreamFactory.getSchemaStream(OBR_DEPLOYMENT_PLAN_XSD);

        SAXSource planSchemaSource = new SAXSource(new InputSource(planSchemaStream));
        SAXSource m2SchemaSource = new SAXSource(new InputSource(m2SchemaStream));
        SAXSource urlSchemaSource = new SAXSource(new InputSource(urlSchemaStream));
        SAXSource obrSchemaSource = new SAXSource(new InputSource(obrSchemaStream));
        Source[] sources = new Source[] {
                    planSchemaSource,
                    m2SchemaSource,
                    urlSchemaSource,
                    obrSchemaSource};
        this.unmarshallerSchema = sf.newSchema(sources);
    }

}
