/**
 * OW2 Util
 * Copyright (C) 2008 SERLI
 * Contact: easybeans@objectweb.org
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
* USA
*
* --------------------------------------------------------------------------
* $Id: RepositoryDataReaderImpl.java 4389 2008-12-15 13:48:57Z alitokmen $
* --------------------------------------------------------------------------
*/


package org.ow2.util.plan.reader.repository;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;

import javax.xml.XMLConstants;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.transform.sax.SAXSource;
import javax.xml.validation.SchemaFactory;

import org.ow2.util.plan.bindings.repository.Repositories;
import org.ow2.util.plan.reader.AbsReader;
import org.ow2.util.plan.reader.ReaderException;
import org.xml.sax.InputSource;

/**
 * Implementation for the IRepositoryDataReader interface.
 * @author Mickaël LEDUQUE
 */
public class RepositoryDataReaderImpl extends AbsReader implements IRepositoryDataReader {

    /**
     * XML binding classes used by RepositoryDataReaders.
     */
    private final static Class[] REPOSITORIES_CLASSES = {
        org.ow2.util.plan.bindings.repository.ObjectFactory.class
    };

    /**
     * Location of the schema resource.
     */
    private final static String REPOSITORY_XSD = "repositories-1.0.xsd";

    /**
     * No-args constructor.
     * @throws JAXBException when the unmarshaller or schema build failed.
     */
    public RepositoryDataReaderImpl() throws Exception {
        super();
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected void initUnmarshallClasses() {
        unmarshallClasses = REPOSITORIES_CLASSES;
    }

    /**
     * {@inheritDoc}
     */
    public Repositories readRepositories(final File file) throws IOException,
                                ReaderException {
        JAXBElement jaxbElement = null;
        try {
            jaxbElement = getRootElement(file);
        } catch (JAXBException e) {
            throw new ReaderException(e);
        }

        Object elementValue = jaxbElement.getValue();
        if (elementValue instanceof Repositories) {
            return (Repositories)elementValue;
        } else {
            throw new ReaderException("Bad root element for repository file.");
        }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected void buildUnmarshallerSchema() throws Exception {
        SchemaFactory sf = SchemaFactory.newInstance(XMLConstants.W3C_XML_SCHEMA_NS_URI);
        // Build the schema for validation.
        InputStream is = this.schemaStreamFactory.getSchemaStream(REPOSITORY_XSD);
        if (is == null) {
            throw new Exception("Cannot read Repository XSD resource");
        }
        SAXSource ss = new SAXSource(new InputSource(is));

        this.unmarshallerSchema = sf.newSchema(ss);
    }
}
